!!****if* source/physics/Hydro/HydroMain/unsplit/Anisotropic_MHD/Hydro_data
!!
!! NAME
!!
!!  Hydro_data
!!
!!
!! SYNOPSIS
!!
!!  MODULE Hydro_data()
!!
!!
!! ARGUMENTS
!!
!!
!! DESCRIPTION
!!
!!  This stores data that are specific to the Unsplit Anisotropic_MHD unit.
!!
!!***
 
 
Module Hydro_data

#include "Flash.h"
#include "constants.h"
#include "UHD.h"

  logical, save :: hy_killdivb,          hy_E_modification,        &
                   hy_fluxCorrect,       hy_charLimiting,          &
                   hy_energyFixSwitch,   hy_useGravity,            &
                   hy_entropy,           hy_forceHydroLimit,       &
                   hy_fullyLimit,        hy_useResistivity,        &
                   hy_shockDetectOn,     hy_flattening,            &
                   hy_facevar2ndOrder,   hy_shockInstabilityFix,   &      
                   hy_ContactSteepening, hy_EOSforRiemann,         &
                   hy_gravConsv,         hy_useGravHalfUpdate,     &
                   hy_upwindTVD,         hy_use_avisc,             &
                   hy_useGravPotUpdate,  hy_updateHydroFluxes,     &
                   hy_useDiffuse,        hy_updateMagneticFields,  &
                   hy_useAnisotropicViscosity,                     &
                   hy_useAnisotropicConductivity,                  &
                   hy_useAnisotropicResistivity

  integer, save :: hy_geometry, hy_limiter, hy_irenorm
  integer, save :: hy_unsplitEosMode = MODE_DENS_EI
  integer, save :: hy_prol_method, hy_RiemannSolver,hy_RiemannSolverLoc, hy_entropyFixMethod
  integer, parameter :: hy_numXN = NSPECIES+NMASS_SCALARS

  character(len=MAX_STRING_LENGTH), save :: hy_limiter_str,      &
                                            hy_prol_method_str,  &
                                            hy_RiemannSolver_str,&
                                            hy_entropyFixMethod_str

  real, save :: hy_cfl
  real, save :: hy_cfl_original
  real, save :: hy_tiny=1.e-32
  real, save :: hy_Rconst, hy_NA, hy_kB
  real, save :: hy_eswitch
  real, save :: hy_smalldens
  real, save :: hy_smallpres
  real, save :: hy_maxMagDiff
  real, save :: hy_LimitedSlopeBeta
  real, save :: hy_cvisc

  ! System of units used
  character(4), save :: hy_units

  ! Everybody should know these!
  integer, save :: hy_numProcs, hy_myPE

  ! Constants for non-dimensionalization
  real, save :: hy_xref
  real, save :: hy_tref
  real, save :: hy_dref
  real, save :: hy_vref
  real, save :: hy_pref
  real, save :: hy_eref
  real, save :: hy_qref
  real, save :: hy_bref
  real, save :: hy_gref
  real, save :: hy_mref
  real, save :: hy_nref
  real, save :: hy_kref

  ! Order of Accuracy
  ! hy_order=1 ==> First order Godunov
  ! hy_order=2 ==> Muscl-Hancock
  ! hy_order=3 ==> PPM
  ! hy_order=5 ==> WENO: Note when using WENO, users should use 6 guardcells 
  !                      along with increased sizes of nxb, nyb, and nzb that are
  !                      larger than 2*NGUARD=12 at least.
  integer, save :: hy_order, hy_order_original, hy_transOrder
  integer,save  :: hy_gcMaskSize=NUNK_VARS+NDIM*NFACE_VARS
  logical,dimension(NUNK_VARS+NDIM*NFACE_VARS),save :: hy_gcMask

  ! Transport parameters
  logical, save :: hy_use_Braginskii
  real, save :: hy_viscosity_0
  real, save :: hy_viscosity_1
  real, save :: hy_viscosity_2
  real, save :: hy_viscosity_3
  real, save :: hy_viscosity_4
  real, save :: hy_conductivity_para
  real, save :: hy_conductivity_perp
  real, save :: hy_conductivity_caret
  real, save :: hy_resistivity_para
  real, save :: hy_resistivity_perp
  real, save :: hy_resistivity_Hall

  !Parameters for calculating time step
  real, save :: hy_max_flux
  real, dimension(10), save :: hy_flux_dt
  real, save :: hy_max_viscosity
  real, save :: hy_max_conductivity
  real, save :: hy_max_resistivity

  ! Constants for Braginskii equations
  logical, save :: hy_Braginskii_Z_dependence
  real, save :: hy_proton_mass
  real, save :: hy_pe_mass_ratio
  real, save :: hy_CoulombLog_const1
  real, save :: hy_CoulombLog_const2
  real, save :: hy_Pi_units
  real, save :: hy_Pi_coeff
  real, save :: hy_K_units
  real, save :: hy_K_coeff
  real, save :: hy_eta_units
  real, save :: hy_eta_coeff
  real, save :: hy_delta_coeff



End Module Hydro_data
