!!****if* source/physics/Hydro/HydroMain/unsplit/Anisotropic_MHD/Hydro_init
!!
!! NAME
!!
!!  Hydro_init
!!
!!
!! SYNOPSIS
!!
!!  Hydro_init(integer(IN) :: myPE)
!!  
!!
!! DESCRIPTION
!! 
!!  This routine initializes unit scope variables which are typically the runtime parameters.
!!  The routine must be called once by Driver_initFlash.F90 first. Calling multiple
!!  times will not cause any harm but is unnecessary.
!!
!! ARGUMENTS
!!
!!  myPE - local processor number
!!
!!***

Subroutine Hydro_init(myPE)

  use Hydro_data, ONLY : hy_units, hy_killdivb, hy_cfl, hy_Rconst, &
                         hy_xref, hy_vref, hy_dref, &
                         hy_mref, hy_tref, hy_eref, hy_nref, &
                         hy_pref, hy_gref, hy_qref, hy_kref, &
                         hy_bref, hy_units,hy_E_modification,&
                         hy_limiter, hy_limiter_str,&
                         hy_geometry, hy_fluxCorrect,&
                         hy_charLimiting, hy_eswitch, hy_energyFixSwitch,&
                         hy_order, hy_smalldens,hy_smallpres,hy_useGravity,&
                         hy_irenorm, hy_entropy,&
                         hy_gcMaskSize, hy_gcMask, hy_myPE, hy_numProcs, &
                         hy_prol_method_str, hy_prol_method, &
                         hy_RiemannSolver_str,hy_RiemannSolver,&
                         hy_forceHydroLimit,hy_LimitedSlopeBeta, hy_facevar2ndOrder, &
                         hy_fullyLimit, hy_cfl_original, hy_shockDetectOn,&
                         hy_shockInstabilityFix, &
                         hy_entropyFixMethod_str,hy_entropyFixMethod,&
                         hy_useDiffuse, hy_updateHydroFluxes, &
                         hy_updateMagneticFields, hy_ContactSteepening,&
                         hy_EOSforRiemann, hy_flattening, hy_transOrder,&
                         hy_order_original, hy_gravConsv,hy_useGravHalfUpdate,&
                         hy_upwindTVD, hy_cvisc, hy_use_avisc, hy_useGravPotUpdate,&
                         hy_proton_mass, hy_pe_mass_ratio, &
                         hy_CoulombLog_const1, hy_CoulombLog_const2, &
                         hy_Pi_units, hy_K_units, hy_eta_units, &
                         hy_Pi_coeff, hy_K_coeff, hy_eta_coeff, hy_delta_coeff, &
                         hy_use_Braginskii, hy_Braginskii_Z_dependence, &
                         hy_viscosity_0, hy_viscosity_1, hy_viscosity_2, &
                         hy_viscosity_3, hy_viscosity_4, hy_conductivity_para, &
                         hy_conductivity_perp, hy_conductivity_caret, &
                         hy_resistivity_para, hy_resistivity_perp, &
                         hy_resistivity_Hall, &
                         hy_useAnisotropicViscosity, &
                         hy_useAnisotropicConductivity, &
                         hy_useAnisotropicResistivity, &
                         hy_max_flux, hy_flux_dt, &
                         hy_max_conductivity, &
                         hy_max_viscosity, &
                         hy_max_resistivity, hy_NA, hy_kB

  use Driver_interface,            ONLY : Driver_abortFlash
  use RuntimeParameters_interface, ONLY : RuntimeParameters_get, &
                                          RuntimeParameters_mapStrToInt
  use PhysicalConstants_interface, ONLY : PhysicalConstants_get
  use Logfile_interface,           ONLY : Logfile_stampMessage
  use Grid_interface,              ONLY : Grid_getNumProcs, &
                                          Grid_setFluxHandling


  implicit none

#include "constants.h"
#include "Flash.h"
#include "UHD.h"

  integer, intent(IN) :: myPE
  character(len=MAX_STRING_LENGTH) :: str_geometry
  real :: hy_electron_mass, init_flux_dt

!!$  !! This official release does not support the 3D implementations.
!!$  if (NDIM == 3) then
!!$     call Driver_abortFlash&
!!$          ("[Hydro_init]The 3D Unsplit Staggered Mesh MHD implemetation "// &
!!$           "is not supported in this official release. "// &
!!$           "Please contact Dr. Dongwook Lee for more information.  "//&
!!$           " Email: dongwook@flash.uchicago.edu")
!!$  endif


  ! Everybody should know these
  hy_myPE = myPE
  call Grid_getNumProcs(hy_numProcs)

  call RuntimeParameters_get("cfl", hy_cfl)
  hy_cfl_original = hy_cfl

  call RuntimeParameters_get("UnitSystem",          hy_units)
  call RuntimeParameters_get("killdivb",            hy_killdivb)
  call RuntimeParameters_get("order",               hy_order)
  hy_order_original = hy_order
  call RuntimeParameters_get("transOrder",          hy_transOrder)
  if ((NGUARD <= 4) .and. (hy_order > 3)) then
     call Driver_abortFlash&
          ("[Hydro_init]: Hydro requires more guardcells for the given hy_order method.")
  endif

#ifdef FLASH_GRID_PARAMESH
  if ((NGUARD > 4) .and. (NXB < 2*NGUARD)) then
     call Driver_abortFlash&
          ("[Hydro_init]: Hydro requires larger NXB, etc. for the given number of guardcells.")
  endif
#endif

  call RuntimeParameters_get("E_modification",      hy_E_modification)
  call RuntimeParameters_get("entropy",             hy_entropy)
  call RuntimeParameters_get('entropyFixMethod',    hy_entropyFixMethod_str)
  call RuntimeParameters_get("charLimiting",        hy_charLimiting)
  call RuntimeParameters_get("eintSwitch",          hy_eswitch)
  call RuntimeParameters_get("slopeLimiter",        hy_limiter_str)
  call RuntimeParameters_get("energyFix",           hy_energyFixSwitch)
  call RuntimeParameters_get("smlrho",              hy_smalldens)
  call RuntimeParameters_get("smallp",              hy_smallpres)
  call RuntimeParameters_get('irenorm',             hy_irenorm)
  call RuntimeParameters_get('prolMethod',          hy_prol_method_str)
  call RuntimeParameters_get('RiemannSolver',       hy_RiemannSolver_str)
  call RuntimeParameters_get('ForceHydroLimit',     hy_forceHydroLimit)
  call RuntimeParameters_get('facevar2ndOrder',     hy_facevar2ndOrder)
  call RuntimeParameters_get('shockDetect',         hy_shockDetectOn)
  call RuntimeParameters_get('fullyLimit',          hy_fullyLimit)
  call RuntimeParameters_get('shockInstabilityFix', hy_shockInstabilityFix)
  call RuntimeParameters_get("use_steepening",      hy_ContactSteepening)
  call RuntimeParameters_get("EOSforRiemann",       hy_EOSforRiemann)
  call RuntimeParameters_get("use_flattening",      hy_flattening)
  call RuntimeParameters_get("use_upwindTVD",       hy_upwindTVD)
  call RuntimeParameters_get("use_avisc",           hy_use_avisc)
  call RuntimeParameters_get("cvisc",               hy_cvisc)
  call RuntimeParameters_get("use_GravPotUpdate",   hy_useGravPotUpdate)

  !! Hydro limit B=0 -----------------------------------------------------------
  if (hy_forceHydroLimit) then
     if (hy_killdivb) then
        hy_killdivb = .false.
        if (myPE .EQ. MASTER_PE) print*, &
             "[Hydro_init] Pure hydro mode is chosen and hy_killdivb is turned off."
        call Logfile_stampMessage &
             (myPE,'[Hydro_init] Pure hydro mode: hy_killdivb is turned off.')
     endif
     if (hy_facevar2ndOrder) then
        hy_facevar2ndOrder = .false.
        if (myPE .EQ. MASTER_PE) print*, &
             "[Hydro_init] Pure hydro mode is chosen and hy_facevar2ndOrder is turned off."
        call Logfile_stampMessage &
             (myPE,'[Hydro_init] Pure hydro mode: hy_facevar2ndOrder is turned off.')
     endif
  endif


  !! Second order advancement in face-centered B fields ------------------------
  if ((NDIM == 1) .and. (hy_facevar2ndOrder)) then
     hy_facevar2ndOrder = .false.
     if (myPE .EQ. MASTER_PE) print*, &
          "[Hydro_init] 1D calculation doesn't need facevar2ndOrder and it's now turned off."
  endif

  !! Consideration on upwinding scheme (CTU) or donor cell scheme --------------
  if (hy_fullyLimit) then
     if ((NDIM == 2) .and. (hy_cfl > 0.5)) then
        call Driver_abortFlash&
          ("[Hydro_init]: A full limiting on transverse differencing is chosen."//&
           "'The cfl number should be lower than 0.5'")
     elseif ((NDIM == 3) .and. (hy_cfl > 0.33)) then
        call Driver_abortFlash&
          ("[Hydro_init]: A full limiting on transverse differencing is chosen."//&
           "'The cfl number should be lower than 0.33'")
     endif
  endif


  !! Gravity -------------------------------------------------------------------
  hy_useGravity = .false.
#ifdef GRAVITY
  call RuntimeParameters_get("useGravity", hy_useGravity)
  if (hy_useGravity) then
     call RuntimeParameters_get("use_gravHalfUpdate", hy_useGravHalfUpdate)
     if (hy_useGravHalfUpdate) then
        call RuntimeParameters_get("use_gravConsv",      hy_gravConsv)
     else
        hy_gravConsv = .false.
     endif
  endif
#endif

  !! Non-ideal diffusions ------------------------------------------------------

  call RuntimeParameters_get("useAnisotropicResistivity",  hy_useAnisotropicResistivity)
  call RuntimeParameters_get("useAnisotropicViscosity",    hy_useAnisotropicViscosity)
  call RuntimeParameters_get("useAnisotropicConductivity", hy_useAnisotropicConductivity)

  hy_useDiffuse = .false.
  if (hy_useAnisotropicResistivity .or. &
      hy_useAnisotropicViscosity   .or. &
      hy_useAnisotropicConductivity) then
     hy_useDiffuse = .true.
  endif

  !! Testing Switches ----------------------------------------------------------

  call RuntimeParameters_get("updateHydroFluxes",    hy_updateHydroFluxes)
  call RuntimeParameters_get("updateMagneticFields", hy_updateMagneticFields)
  if (.not. hy_updateMagneticFields) then
     hy_killdivb = .false.
  endif

  !! DivB=0 constraint ---------------------------------------------------------
  if ((NDIM == 1) .OR. (hy_order == 1)) then
     hy_killdivb=.false.
     hy_energyFixSwitch=.false.
     if (myPE .EQ. MASTER_PE) print*, &
          "[Hydro_init] Parameters killdivb, and energyFix are not "// &
          "required and now they are turned off."
     call Logfile_stampMessage &
          (myPE,'[Hydro_init] Parameters killdivb, and energyFix are now turned off')
  endif


  !! Entropy Fix ---------------------------------------------------------------
  if (hy_entropy) then
     if(trim(hy_entropyFixMethod_str) == "HartenHyman" .or. &
        trim(hy_limiter_str) == "hartenhyman") then
        hy_entropyFixMethod = HARTENHYMAN
     elseif(trim(hy_entropyFixMethod_str) == "Harten" .or. &
            trim(hy_limiter_str) == "harten") then
        hy_entropyFixMethod = HARTEN
     endif
  endif


  !! Slope Limiter -------------------------------------------------------------
  if(trim(hy_limiter_str) == "minmod" .or. &
     trim(hy_limiter_str) == "MINMOD") then
     hy_limiter = MINMOD
  else if(trim(hy_limiter_str) == "mc" .or. &
          trim(hy_limiter_str) == "MC" ) then
     hy_limiter = MC
  else if (trim(hy_limiter_str) == "hybrid" .or. &
           trim(hy_limiter_str) == "HYBRID" ) then
     hy_limiter = HYBRID
  else if (trim(hy_limiter_str) == "vanLeer" .or. &
           trim(hy_limiter_str) == "VANLEER") then
     hy_limiter = VANLEER
  else if (trim(hy_limiter_str) == "limited" .or. &
           trim(hy_limiter_str) == "LIMITED") then
     hy_limiter = LIMITED
     call RuntimeParameters_get('LimitedSlopeBeta', hy_LimitedSlopeBeta)
  else
     call Driver_abortFlash&
          ("[Hydro_init]: The hy_limter of unknown type! It should be one of" // &
           "'minmod','mc', 'vanLeer','hybrid' or 'limited'.")
  end if


  !! Prolongation Method for divergence-free face-centered B fields ------------
  if(trim(hy_prol_method_str) == "injection_prol" .or. &
     trim(hy_prol_method_str) == "INJECTION_PROL" ) then
     hy_prol_method = INJECTION_PROL
  elseif (trim(hy_prol_method_str) == "balsara_prol" .or. &
          trim(hy_prol_method_str) == "BALSARA_PROL" ) then
     hy_prol_method = BALSARA_PROL
  else
     call Driver_abortFlash&
          ("[Hydro_init]: The prolongation method is of unknown type: " // &
           "Please choose one of 'injection_prol(2D & 3D)' or 'balsara_prol(3D)'.")
  endif


  !! Riemann Solver ------------------------------------------------------------
  if(trim(hy_RiemannSolver_str) == "Roe" .or. &
     trim(hy_RiemannSolver_str) == "roe" .or. &
     trim(hy_RiemannSolver_str) == "ROE" ) then
     hy_RiemannSolver = ROE
  elseif (trim(hy_RiemannSolver_str) == "hll" .or. &
          trim(hy_RiemannSolver_str) == "HLL" ) then
     hy_RiemannSolver = HLL
  elseif (trim(hy_RiemannSolver_str) == "hllc" .or. &
          trim(hy_RiemannSolver_str) == "HLLC" ) then
     hy_RiemannSolver = HLLC
  elseif (trim(hy_RiemannSolver_str) == "hlld" .or. &
          trim(hy_RiemannSolver_str) == "HLLD" ) then
     hy_RiemannSolver = HLLD
  elseif (trim(hy_RiemannSolver_str) == "marquina" .or. &
          trim(hy_RiemannSolver_str) == "Marquina" ) then
     hy_RiemannSolver = MARQ
  elseif (trim(hy_RiemannSolver_str) == "LocalLaxFriedrichs" .or. &
          trim(hy_RiemannSolver_str) == "llf"  .or. &
          trim(hy_RiemannSolver_str) == "LLF" ) then
     hy_RiemannSolver = LLF
  elseif (trim(hy_RiemannSolver_str) == "marquinaMod" .or. &
          trim(hy_RiemannSolver_str) == "MarquinaMod" ) then
     hy_RiemannSolver = MMAR
  elseif (trim(hy_RiemannSolver_str) == "hll7" .or. &
          trim(hy_RiemannSolver_str) == "HLL7" ) then
     hy_RiemannSolver = HLL7
  else
     call Driver_abortFlash&
          ("[Hydro_init]: The Riemann Solver is of unknown type: " // &
           "Options are Roe, HLL, HLLC, HLLD, HLL7,"//&
           "Marquina, MarquinaMod, or LLF.")
  endif

  !! Geometry ------------------------------------------------------------------
  call RuntimeParameters_get("geometry", str_geometry)
  call RuntimeParameters_mapStrToInt(str_geometry, hy_geometry)
  if (hy_geometry .NE. CARTESIAN )  then
     call Driver_abortFlash&
          ("[Hydro_init]: Curvilinear coordinates are not supported in StaggeredMesh MHD solver")
  endif


  !! Flux conservation on AMR grid ---------------------------------------------
  call RuntimeParameters_get("flux_correct", hy_fluxCorrect)
  if (NDIM > 1) then
     if (hy_fluxCorrect) then
        call Grid_setFluxHandling('consv_flux_densities')
     end if
  end if

  call PhysicalConstants_get("ideal gas constant", hy_Rconst)
  call PhysicalConstants_get("Avogadro", hy_NA)
  call PhysicalConstants_get("Boltzmann", hy_kB)

  !! System units --------------------------------------------------------------
  hy_xref = 1.0
  hy_vref = 1.0
  hy_dref = 1.0

  hy_mref = hy_xref*hy_vref
  hy_tref = hy_xref/hy_vref
  hy_eref = hy_vref*hy_vref
  hy_nref = hy_dref*hy_vref*hy_xref
  hy_pref = hy_dref*hy_vref*hy_vref
  hy_gref = hy_vref*hy_vref/hy_xref

  hy_qref = hy_vref*hy_vref/hy_Rconst
  hy_kref = hy_dref*hy_vref*hy_xref*hy_Rconst

  if ( hy_units == "SI" .or. hy_units == "si" ) then
    hy_bref = hy_vref*sqrt(4.0*PI*hy_dref*1.e-7)
  else if ( hy_units == "CGS" .or. hy_units == "cgs" ) then
    hy_bref = hy_vref*sqrt(4.0*PI*hy_dref)
  else
    hy_bref = hy_vref*sqrt(hy_dref)
  end if

  !! Transport Units -----------------------------------------------------------

  call RuntimeParameters_get("use_Braginskii",          hy_use_Braginskii)
  call RuntimeParameters_get("Braginskii_Z_dependence", hy_Braginskii_Z_dependence)
  call RuntimeParameters_get("viscosity_0",             hy_viscosity_0)
  call RuntimeParameters_get("viscosity_1",             hy_viscosity_1)
  call RuntimeParameters_get("viscosity_2",             hy_viscosity_2)
  call RuntimeParameters_get("viscosity_3",             hy_viscosity_3)
  call RuntimeParameters_get("viscosity_4",             hy_viscosity_4)
  call RuntimeParameters_get("conductivity_para",       hy_conductivity_para)
  call RuntimeParameters_get("conductivity_perp",       hy_conductivity_perp)
  call RuntimeParameters_get("conductivity_caret",      hy_conductivity_caret)
  call RuntimeParameters_get("resistivity_para",        hy_resistivity_para)
  call RuntimeParameters_get("resistivity_perp",        hy_resistivity_perp)
  call RuntimeParameters_get("resistivity_Hall",        hy_resistivity_Hall)

  hy_max_viscosity = 0.
  if( hy_viscosity_0 > hy_max_viscosity ) hy_max_viscosity = hy_viscosity_0
  if( hy_viscosity_1 > hy_max_viscosity ) hy_max_viscosity = hy_viscosity_1
  if( hy_viscosity_2 > hy_max_viscosity ) hy_max_viscosity = hy_viscosity_2
  if( hy_viscosity_3 > hy_max_viscosity ) hy_max_viscosity = hy_viscosity_3
  if( hy_viscosity_4 > hy_max_viscosity ) hy_max_viscosity = hy_viscosity_4
  hy_max_conductivity = 0.
  if( hy_conductivity_para  > hy_max_conductivity ) hy_max_conductivity = hy_conductivity_para
  if( hy_conductivity_perp  > hy_max_conductivity ) hy_max_conductivity = hy_conductivity_perp
  if( hy_conductivity_caret > hy_max_conductivity ) hy_max_conductivity = hy_conductivity_caret
  hy_max_resistivity = 0.
  if( hy_resistivity_para > hy_max_resistivity ) hy_max_resistivity = hy_resistivity_para
  if( hy_resistivity_perp > hy_max_resistivity ) hy_max_resistivity = hy_resistivity_perp
  !if( hy_resistivity_Hall > hy_max_resistivity ) hy_max_resistivity = hy_resistivity_Hall

  call PhysicalConstants_get ("proton mass", hy_proton_mass)
  call PhysicalConstants_get ("electron mass", hy_electron_mass)
  hy_pe_mass_ratio = hy_proton_mass/hy_electron_mass

  if ( hy_units == "SI" .or. hy_units == "si" ) then
     hy_CoulombLog_const1 = 15.9
     hy_CoulombLog_const2 = 21.6
     hy_delta_coeff = 4.85e16
     hy_Pi_units = 1.0 !already in  kg/m-s
     hy_Pi_coeff = 2.309e-16
     hy_K_units = 1.0 !already in W/m-K
     hy_K_coeff = 4.170e12*1.381e-23
     hy_eta_units = 1.0 !already in Ohm-m
     hy_eta_coeff = 128.6
  else if ( hy_units == "CGS" .or. hy_units == "cgs" ) then
     hy_CoulombLog_const1 = 9.0
     hy_CoulombLog_const2 = 14.6
     hy_delta_coeff = 4.85e6
     hy_Pi_units = 10. !convert kg/m-s to poise
     hy_Pi_coeff = 2.309e-15
     hy_K_units = 1.e5 !convert W/m-K to erg/cm-sec-K
     hy_K_coeff = 4.170e10*1.381e-16
     hy_eta_units = 1.e-9/9. !convert Ohm-m to sec
     hy_eta_coeff = 1.431e-8
  else
     hy_CoulombLog_const1 = 1.0
     hy_CoulombLog_const2 = 1.0
     hy_delta_coeff = 1.0
     hy_Pi_units = 1.0
     hy_Pi_coeff = 1.0
     hy_K_units = 1.0
     hy_K_coeff = 1.0
     hy_eta_units = 1.0
     hy_eta_coeff = 1.0
  end if

  ! time step limits due to flux
  call RuntimeParameters_get("max_flux",       hy_max_flux)
  call RuntimeParameters_get("dtinit",         init_flux_dt)
  hy_flux_dt = init_flux_dt !apply scalar to array

  !! Allow selective guardcell fill calls ---------------------------------------
  hy_gcMaskSize = NUNK_VARS+NDIM*NFACE_VARS
  hy_gcMask = .TRUE.
#ifdef DIVB_VAR
  hy_gcMask(DIVB_VAR) = .FALSE.
#endif
#ifdef MAGP_VAR
  hy_gcMask(MAGP_VAR) = .FALSE.
#endif
#ifdef TOTP_VAR
  hy_gcMask(TOTP_VAR) = .FALSE.
#endif
#ifdef BETA_VAR
  hy_gcMask(BETA_VAR) = .FALSE.
#endif
#ifdef VECZ_VAR
  hy_gcMask(VECZ_VAR) = .FALSE.
#endif
#ifdef CURX_VAR
  hy_gcMask(CURX_VAR) = .FALSE.
#endif
#ifdef CURY_VAR
  hy_gcMask(CURY_VAR) = .FALSE.
#endif
#ifdef CURZ_VAR
  hy_gcMask(CURZ_VAR) = .FALSE.
#endif

#if NSPECIES == 1
#ifdef SPECIES_BEGIN
  hy_gcMask(SPECIES_BEGIN) = .FALSE.
#endif
#endif

End Subroutine Hydro_init

